(* euvec.m *)

(* Created by Rolf Sulanke *)

(* Started January 15, 1999. *)
(* Finished February 24, 1999. Revised May 25, 1999 *)
(* Revised October 4, 2000 *)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(* This package contains definitions and procedures for n-dimensional
euclidean geometry in terms of vector algebra. Applications, explanations,
and certain more special concepts are included in the notebook eusphere.nb.*)

(*Most of the concepts depend on the dimension dim, which must be defined in
the  Global Context! *)

(* Keywords: vector objects, null vector, basis vectors, random vector, 
normalization, cross product, stereographic projection, inverse stereographic 
projection, hyperplanes, spheres, spheres through four points, hyperspheres, 
n-spheres, orthoframes in the 4-dimensional euclidean vector space, 
Erhard Schmidt's orthogonalization. *)


BeginPackage["euvec`"]


(*The usages *)

nul::usage = "nul[d:dim] is the zero vector of length d; dim must be defined 
first.";


stb::usage = "stb[i, d:dim] is the i-th standard basis vector, 1 <= i <= d; 
dim must be defined first.";

vec::usage = "vec[x, n:dim] is the n-vector with components x[i].";

randomv::usage = "randomv[n:dim] is a random n-vector with values in the positive unit
n-cube."

smoothing::usage = "smoothing[a, b] is a listable function setting small (<
b: 10^-10) arguments to zero; for b = 1 a is returned: smoothing[a, 1] = a."  \
 

rank::usage = "rank[m] calculates the rank of the rectangular matrix m of a \
not
 too large order (< 10000)."

dotnorme::usage = "dotnorme[b] yields the unit vector with respect to the Dot
product in direction of b, b!=o." 

cross::usage = "cross[v] gives the vector product of a list v of 
n - 1 vectors of the euclidean space of dimension n ."

outzero::usage = "outzero[b_List] is the sublist of the vectors b[[i]] !=
nul[]."

sterproj::usage ="sterproj is the stereographical projection of the unit 
n-sphere onto the euclidean (equatorial) n-space. WARNING: The result is correct
only for arguments of norm 1 !";

invsterproj::usage="invsterproj is the inverse of the general euclidean 
stereographic projection sterproj.";

hyperplane::usage ="hyperplane[plist] returns the normal vector n and the
distance p from the origin of  the hyperplane  of the N-dimensional euclidean
space spanned by the N points in general position listed in plist; plist must
be a square  matrix of order N."

unit::usage = "unit[n][s_List] generates a parameter representation of the \
\
unit n-sphere with centre o in the euclidean (n+1)-space. The parameters \
are
the first n elements of the list s, which must have length at least n. The
parameter ranges  0 <s[[1]]<=2Pi, -Pi/2 <= s[[j]] <= Pi/2 give the whole
n-sphere."

sphericalreflection::usage = "sphericalreflection[x, a, r] gives the point in inversion with x \
with 
respect to the hypersphere with center a and radius r."

esorthonorm::usage = "esorthonorm[b_List, opts] is the Erhard Schmidt
orthogonalization for the list b of vectors with respect to a positive
semi-definite scalar product which can be specified as an option; the
default is Dot."

innerprod::usage = "innerprod is an option for esorthonorm and orthonorm;the
default is Dot (esorthonorm), resp. pssp (orthonorm)."

normed::usage = "normed is an option for esorthonorm and orthonorm which specifies whether
the vectors produced by esorthonorm or orthonorm are normed, or only orthogonal;the default
is True."

print::usage = "print is an option for esorthonorm and orthonorm which specifies whether
the (unnormed) vectors produced by esorthonorm or orthonorm are printed during the
orthogonalization procedure;the default is False."

neglect::usage = "neglect is an option for esorthonorm and orthonorm which specifies whether
vectors with components or scalar square less than 10^neglect
produced by esorthonorm  or orthonorm during the orthogonalization procedure are set to
zero;the default is neglect -> -10.For neglect -> 0 no neglecting takes place."

renorm::usage = 
  "renorm[b] for a given matrix b  is the matrix with  normed vectors 
b[[i]]/|b[[i]]|, where |x| denotes the usual euclidean norm of the vector \
x. \
Before norming the zero vectors of b are deleted."

orthobase::usage = "orthobase[v] is an orthonormal basis of the finite-dimensional euclidean vector space V 
the last vector of which is proportional to the given vector v;
for v = nullvector orthobase returns the standard basis."


(* Only for the case dim = 4 :*)

ortho1::usage = "u = ortho1[v, w] is a unit vector of the Euclidean 4-space 
orthogonal to the 2-space spanned by v,w.";


orthoerg::usage = "orthoerg[v, w] yields two orthonormal 4-vectors orthogonal \
to the span of v, w." ;

orthoframe::usage = "orthoframe[v] is an orthonormal frame of the \
\
4-dimensional euclidean space with 4th vector v/Sqrt[v.v]; v must be a list \
of
Length 4,  v != 0. (May be replaced by orthobase[v])";

subsphere::usage = "subsphere[n,p][v,w] is the parameter  representation of the
2-sphere in the euclidean 4-space, in which  the hyperplane n.x=p intersects
the unit 3-sphere. 
n is the normed normal vector of the hyperplane,  
p,-1<p<1, is the distance  of the hyperplane from the origin, 
v (0 <= v < 2Pi), w  (-Pi/2 <= w <= Pi/2), are the parameters for the
2-sphere.  Its radius is Sqrt[1-p^2], and its center is n*p."

tg4frame::usage = "tg4frame[s,t,u] is an orthonormal moving frame in the
euclidean 4-space such that the fourth vector is the radius vector (= normal
vector) of the unit 3-sphere with geographic coordinates s,t,u."

subspheremf::usage ="subspheremf[s,t,u,p][v,w] is the parameter  representation of
the 2-sphere in the euclidean 4-space, in which  the hyperplane n.x=p
intersects the unit 3-sphere. 
s,t,u are stereographical coordinates on the unit 3-sphere, 
n=n[s,t,u] is the normed normal vector of the hyperplane, 
p, -1<p<1, is the distance of the  hyperplane from the origin, 
v  (0 <= v < 2Pi), w  (-Pi/2 <= w <= Pi/2), are the parameters for the
2-sphere.  Its radius is Sqrt[1-p^2], and its center is n*p."

sphere::usage = "sphere[m,r][u,v] is the parameter representation of 
the sphere with center m and radius r in the euclidean 3-space with geographical 
coordinates u, 0<=u<=2 Pi, v, -Pi/2 <= v <= Pi/2."

sphereplot3D::usage = "sphereplot3D[m,r, u0,u1,v0,v1,opt] is the plot of the 
part of the sphere with center m and radius r, which is defined by u0<=u<=u1, 
v0<=v<=v1; the default is the whole sphere given by the default values u0=0, 
u1=2 Pi, v0 =-Pi/2, v1=Pi/2. The options opt are transferred to
ParametricPlot3D."

Begin["`Private`"]
(* The dimension must be set within the Global Context. *)

dimm := Global`dim;

(* The null vector. *)

nul[d_:dimm] := Table[0, {d}]

(*A general vector *)

vec[x_, n_:dimm] := Table[x[i], {i, n}]

(*A random vector with values in the unit n-cube *)

randomv[n_:dimm]:= Table[Random[],{i,n}]

(* The standard basis.*)

stb[i_Integer, d_:dimm] := IdentityMatrix[d][[i]]
 
(*A function which makes small entities to become zero.*)
  
  
smoothing[a_, b_:10^-10]:=
  Module[{ c= a}, If[b == 1, Return[c], If[Abs[c]< b, c=0]]; c];
Attributes[smoothing]=Listable   
(*rank definition *)  

rank[m_] := Module[{ di = Length[m[[1]]], nsp},
		nsp = NullSpace[m];
		di - Length[nsp]]/;MatrixQ[m]

(*Definition of the stereographic projection of the n-sphere onto the \
\
n-dimensional euclidean space: *)

sterproj[v_List] :=Module[{m=Length[v],l, w},
		l=nul[m];
		w=v[[m]];
		If[smoothing[w - 1]== 0,For[i=1,i<m,i++,l[[i]]=Infinity],
		l=Simplify[ stb[m,m] + (v- stb[m,m])/(1- w)],
		l=Simplify[ stb[m,m] + (v- stb[m,m])/(1- w)]];
		l=Take[l,m-1];
		l]


(*The inverse stereographic projection*)

invsterproj[y_List]:= Module[{n=Length[y]+1,x=Append[y,0], u=y.y},
		x*2/(1+u) +stb[n,n]*(u-1)/(u+1)]

(* Norming a vector *)

dotnorme[b_] := b/Sqrt[b.b]

(* A private version of the Euclidean cross product for arbitrary dimension:*)

cross[b_List] := Module[{d = Length[b] + 1,v},
		v = nul[d];
			For[i=1,i<d,i++,If[Length[b[[i]] ]!= d, 
			Print[b[[i]], " is not a correct argument!"];Return[],
			v[[i]] = Det[Join[b, {stb[i, d]}]]]];
			v[[d]] = Det[Join[b, {stb[d, d]}]];
			Return[v]]

hyperplane[pt_List] := Module[{n,q,p,d=Length[pt], fin},
		q = Table[pt[[i]] - pt[[d]], {i,d - 1}];
			n = cross[q];
	If[Length[n] != d, Goto[fin]];
	If[n === nul[], Print["The points ", pt, " are not in general position!"]; Goto[fin]];
n = n/Sqrt[n.n];
p = n.pt[[1]];		
Return[{n, p}];
	Label[fin];
Return[]]

(* Inversions at hyperspheres *)

sphericalreflection[{Infinity}, a_List,r_:1] := a;
sphericalreflection[x_List, a_List, r_:1] := If[Not[x==a],
a + ((x - a) * r^2)/((x-a).(x-a)),{Infinity},
a + ((x - a) * r^2)/((x-a).(x-a))]

(*A fast Erhard Schmidt orthonormalization procedure; may be applied also for
functional spaces, the inner product of which must be positive semi-definite. *)

Options[esorthonorm] = {innerprod ->Dot,normed -> True, print ->False, 
neglect-> -10}

esorthonorm[m_List, opts___] := 
  Module[
		{ma = m, mb = {}, k = Length[m], rd = Length[m], j = 0, i, ne, prod, pr, 
      bn, sm, aus, h, f},
		{prod, nor, pr, bn} = 
			{innerprod, normed, print, neglect} /. {opts} /. Options[esorthonorm];
    If[Head[m[[1]] ] == List, rd = Length[m[[1]]]];
	f[h_] := smoothing[h, 10^bn];
	sm[aus_] := MapAll[f, aus];
		Label[ne];
    If[Length[mb] >= Min[rd, k], Return[mb]];
		j = j + 1;
    If[j > k, Return[mb]];
			ma[[j]] = sm[Simplify[ma[[j]] -
	Sum[mb[[i]]*prod[ma[[j]], mb[[i]]]/prod[mb[[i]], mb[[i]]], 
	{i, 1, Length[mb]}]]]; 		
If[pr == True, Print[ma[[j]]]];
If[prod[ma[[j]], ma[[j]]] == 0, Goto[ne],
        If[nor == True, 
        AppendTo[mb, sm[Simplify[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]]]]],
				AppendTo[mb, Simplify[ma[[j]]]]],
				If[nor == True,
				AppendTo[mb, sm[Simplify[ma[[j]]/Sqrt[prod[ma[[j]], ma[[j]]]]]]], 			
AppendTo[mb, Simplify[ma[[j]]]]]];		
		Goto[ne]]



(* The renorming procedure (useful for badly conditioned matrices) *)
renorm[b_?MatrixQ]:= Module[{bo = outzero[b],i },
	Table[dotnorme[bo[[i]]], {i, Length[bo]}]]

(* Construction of a special orthonormal basis in 
finite-dimensional euclidean vector spaces. *)

orthobase[vn_] := Module[{d = Length[vn], tt ,to, v}, 
    tt = IdentityMatrix[d];
    If[vn == nul[d], Return[tt]];
    PrependTo[tt, vn];
    to = esorthonorm[tt];
    v = to[[1]];
    tt=Take[to, -(d-1)];
    AppendTo[tt, v];
    Return[tt]]

(* The following definitions work in case dim = 4 only!
 Here the orthonormal complement of two orthonormal vectors in the Euclidean
4-space is defined;
first we find a vector orthonormal to the vectors
v, w: *)

ortho1[v_, w_] := Module[{l, k=1,cr},
Label[cr];
l = Cross[v, w, stb[k,4]];
If[l == nul[4], k=k+1; If[k>4, Return[Print[v,  w, " are linearily
dependent"]]];Goto[cr],
Return[MapAll[smoothing,Simplify[dotnorme[l]]]],
Return[MapAll[smoothing,Simplify[dotnorme[l]]]]]] 

orthoerg[v_List, w_List] := 
Module[{ve, we},
	If[Length[v] != 4 || Length[w] != 4,
	Return[Print[v, " or ", w, " is not a correct argument!"]]];
	ve = ortho1[v, w];
	If[Length[ve]!= 4, Return[]];
	we = MapAll[smoothing,Simplify[dotnorme[Cross[v, w, ve]]]];
	{ve, we}]

(* Definition of a 4-dimensional Euclidean orthoframe with given 
4-th vector nv: *)

orthoframe[nv_List] := Module[{a = nv.nv, k, m = IdentityMatrix[4], om},
	If[a == 0 || Length[nv] != 4, Return[Print[
				"Vector ", nv, ": incorrect argument!"]]]; 
	m[[4]] = nv/Sqrt[a];
	For[k = 1, k <= 4, k++,
		If[nv[[k]] == 0, 
			m[[3]] = stb[k, 4]; om = orthoerg[m[[3]], m[[4]]];
			m[[1]] = om[[1]]; m[[2]] = -om[[2]]; Return[m]]];
	m[[3, 1]] = -m[[4, 2]]; m[[3, 2]] = m[[4, 1]]; 
	m[[3, 3]] = -m[[4, 4]]; m[[3, 4]] = m[[4, 3]];
	om = orthoerg[m[[3]], m[[4]]];
		m[[1]] = om[[1]]; m[[2]] = -om[[2]]; Return[m]
]

subsphere[n_List, p_][v_,w_] := Module[{fr,pa},
	If[Length[n]!= 4 || Abs[p] > 1,	Print[ n, " or ", p,":  incorrect
arguments?"]; Return[]];	
	fr = orthobase[n];
		pa := fr[[4]] *p +
				((fr[[1]]*Cos[v] + fr[[2]]*Sin[v])*Cos[w] +
							fr[[3]]*Sin[w])*Sqrt[1 - p^2];
		Return[pa]]

tg4frame[s_,t_,u_] :={{-Sin[s],Cos[s],0,0},
			{-Cos[s]*Sin[t],-Sin[s]*Sin[t], Cos[t], 0},
		{-Cos[s] Cos[t] Sin[u],-Cos[t] Sin[s] Sin[u],-Sin[t] Sin[u],Cos[u]},{Cos[s] \
\
Cos[t] Cos[u],Cos[t] Cos[u] Sin[s],Cos[u] Sin[t],Sin[u]}}

subspheremf[s_,t_,u_, p_][v_,w_] := Module[{fr,pa},
		fr = tg4frame[s,t,u];
			pa := fr[[4]] *p +
				((fr[[1]]*Cos[v] + fr[[2]]*Sin[v])*Cos[w] +
							fr[[3]]*Sin[w])*Sqrt[1 - p^2];
		Return[pa]]


(* Deleting null vectors of a sequence. *)

outzero[b_List] := Module[{v= {}, l = Length[b]},
			For[i = 1, i<= l, i++, 
				If[Not[smoothing[b[[i]].b[[i]]] == 0],
AppendTo[v, b[[i]]],Null[],AppendTo[v, b[[i]]]]]; 		Return [v]]

 (* old version: 
 outzero[b_List] := Module[{v= {}, l = Length[b]},
			For[i = 1, i<= l, i++, 
				If[Not[b[[i]] ===
nul[Length[b[[i]]]]],AppendTo[v, b[[i]]]]]; 		Return [v]] *)

(*Recursive Definition of the parameter representation of the unit n-sphere*)

(*This definition is independent of a globally defined dimension dim. *)

unit[n_][s_List] := Module[{un,i},
		un[1]={Cos[s[[1]]], Sin[s[[1]]]};
			For[i=2, i<= n, i++,
		un[i]=AppendTo[un[i-1],0]*Cos[s[[i]]]+stb[1+i,i+1]*Sin[s[[i]]]];
				Return[un[n]]]/;Length[s]>=n

(*Spheres in the euclidean 3-space *)

sphere[m_List, r_][u_, v_] := {m[[1]] + r*Cos[u]*Cos[v], 
    m[[2]] + r*Cos[v]*Sin[u], m[[3]] + r*Sin[v]} /; Length[m] == 3
    
sphereplot3D[m_List, r_, u0_:0, u1_:2 Pi,v0_:-Pi/2,v1_:Pi/2, opt___] := 
  ParametricPlot3D[sphere[m,r][u,v],{u,u0,u1},{v,v0,v1},opt]

End[]

Protect[nul,stb, cross, dotnorme, esorthonorm, innerprod,  normed, print,
neglect, renorm,outzero, rank, sterproj, invsterproj, ortho1, orthobase, orthoerg,
orthoframe, randomv,  smoothing, sphere, sphereplot3D, subsphere, subspheremf, 
sphericalreflection, 
tg4frame, hyperplane, unit, vec]



EndPackage[]


