(***********************************************************************
File: liealg.m

This package contains definitions and procedures for Lie algebras of 
matrices, especially for the classical simple Lie algebras.
It is a shortened version of the package liealg.m in the item "Lie Algebras",
http://www.mathsource.com/cgi-bin/msitem?0210-845
Here it is adapted to the application in Moebius geometry.

Author: Rolf Sulanke

Address:
sulanke@mathematik.hu-berlin.de
Homepage:
http://www-irm.mathematik.hu-berlin.de/~sulanke/

For comments, hints,  and corrections the author will be very thankful.

Date: November 20, 1999, Revised Jan 16, 2000

Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.

Hints: 
1. Before importing this package be sure that the numbers
dim (= dimension) and ind (=index) of the pseudo-euclidean vector space are \
defined as constants within the Global context !!!
2. The symbols and definitions in comments appear already in other
context in the item Spheres!
***********************************************************************)




(* Keywords: 
indefinite scalar products, Lie algebra, semisimple Lie algebra,  
compact Lie algebra, orthogonal Lie algebra, pseudo-orthogonal Lie algebra, 
linear Lie algebra, special linear Lie algebra, symplectic Lie algebra, 
orthogonalization, Killing form, commutator, structure constants, center, 
normalization. *) 

BeginPackage["liealg`"]

(*The concepts introduced in this package depend on  the dimension 
dim of the underlying vector space (= the order of the square matrices) and, 
in the pseudo-euclidean case,  the index  ind of the (non-degenerate, 
symmetric) scalar product on this space. dim and ind must be defined in the  
Global Context before importing the package!*) 

(*The usages *) 

(*
vec::usage = "vec[x, n:dim] is the n-vector with components x[i].";

randomv::usage = "randomv[n:dim] is a random n-vector with values in the 
unit n-cube."

smoothing::usage = "smoothing[a, b] is a listable function setting small 
(< b: 10^-10) arguments to zero; for b = 1 a is returned: 
smoothing[a, 1] = a."  *)

(* randommatrix::usage="randommatrix[k, n:dim] is a matrix with n rows and 
k columns, with random elements in the unit intervall." *)

nullmatrix::usage="nullmatrix[k, n] is a matrix with n rows and 
k columns, with all elements zero; nullmatrix[n:dim] is the square 
nullmatrix of order n."

matrix::usage="matrix[a, k, n] is a matrix with n rows and 
k columns, with elements a[i,j]; matrix[a, n:dim] is the square 
nullmatrix of order n with elements a[i,j]."

commute::usage="commute[a,b] is the commutator of a, b; a, b must be 
square matrices of the same order."

basismatrix::usage="basismatrix[i,j,n:dim] is the sqare matrix of order n 
with element 1 at the place (i,j) and all oter elements zero."

glbasis::usage="glbasis[n:dim] is the standard basis of the general linear
Lie algebra gl(n, K) of order n."

cobasis::usage="cobasis[i,j,n:dim][m] is the linear form which yields 
the element m[i,j] of the matrix m."

strconst::usage="strconst[i,j,r,s,u,v,n:dim] are the structure constants 
of the general linear Lie algebra of order n." 

glkilling::usage="glkilling[a,b,n:dim] is the Killing form of gl(n,K)."

zelm::usage="zelm[z, n:dim] is the diagonal matrix of order n with all 
diagonal elements the same numberb z."

matrixoutzero::usage="If b is a sequence of square matrices of order n,
then matrixoutzero[b] is the sequence of all matrices b[[i]], for which 
smoothing[b[[i]]] is not the nullmatrix of order n."

(*
normalize::usage="normalize[v, prod:glkilling] returns the normalized 
vector or matrix v/Sqrt[Abs[v,v]]; if Abs[v,v] = 0 the element v is 
returned." *)

matrixorthonorm::usage="matrixorthonorm[b, prod:glkilling,opts] yields an 
orthonormal (or orthogonal) sequence of non null matrices with respect 
to the symmetric bilinear form prod, which has the same span as b; the 
orthogonalized sequence may contain linear dependent isotropic matrices.
The default options of matrixorthonorm are {innerprod->glkilling,
normed->True, print->False, neglect-> -10}."

matrixinnerprod::usage="matrixinnerprod -> prod is an option for
matrixorthonorm  defining the symmetric bilinear form used in the orthonorming
procedure; its default value is glkilling."
(*
normed::usage="normed is an option for matrixorthonorm; 
the default value normed -> True ensures that orthonorm yields a normed 
sequence. If orthonorm cannot orthogonalize the sequence with this option, 
try normed -> False." 

print::usage="print is an option for matrixorthonorm; print -> True  
prints step by step the not normed elements constructed during 
the orthonorming procedure; its default value is False."

neglect::usage="neglect is an option for matrixorthonorm 
defining the sensibility b^(neglect) of smoothing used in the 
orthonorming procedure; its default value is neglect -> -10.
For neglect -> 0 smoothing is switched off." *)

slbasismatrix::usage="slbasismatrix[i,j,n:dim] is an element of the 
standard basis of the Lie algebra sl(n,K)."

slbasis::usage="slbasis[n:dim] is the 
standard basis of the Lie algebra sl(n,K)."
 
obm::usage="obm[i, j, n:dim] is an element of the 
standard basis of the orthogonal Lie algebra o(n)."

obasis::usage="obasis[n:dim] is the 
standard basis of the orthogonal Lie algebra o(n)."

oelm::usage="oelm[a, n:dim] is the symbolic element of the orthogonal 
Lie algebra o(n) with coefficients a[i,j]."

okilling::usage="okilling[a, b, n:dim] is the Killing form of the 
orthogonal Lie algebra o(n)."

psobm::usage="psobm[i, j, k:ind, n:dim] is an element of the 
standard basis of the pseudo-orthogonal Lie algebra o(n, k)."

psobasis::usage="psobasis[k:ind, n:dim] is the 
standard basis of the pseudo-orthogonal Lie algebra o(n, k)."

psoelm::usage="psoelm[a, k:ind, n:dim] is the symbolic element of the 
pseudo-orthogonal Lie algebra o(n, k) with coefficients a[i,j]."

psokilling::usage="psokilling[a, b, k:ind, n:dim] is the Killing form 
of the pseudo-orthogonal Lie algebra o(n, k)."

alt::usage="alt[n] (n even) is the (scew) matrix of the scalar products 
of the standard basis elements in an n-dimensional symplectic vector 
space."

altprod::usage="altprod[x,y,n] (n even) is the (alternating) scalar 
product of the vectors x,y in an n-dimensional symplectic vector 
space."

spbasis::usage="spbasis[i, j, n] (n even) is an element of the 
standard basis of the symplectic Lie algebra sp(n,K)."

spbasislist::usage="spbasislist[n] (n even) is the 
standard basis of the symplectic Lie algebra sp(n,K)."

spelm::usage="spelm[a, n] (n even) is the symbolic element of the 
symplectic Lie algebra sp(n,K) with coefficients a[i,j]."

spkilling::usage="spkilling[a, b, n] (n even) is the Killing form 
of the symplectic Lie algebra sp(n,K)."



Begin["`Private`"]

(* The dimension and the index must be set within the Global Context. *)

dim := Global`dim;

ind := Global`ind;

(*A general vector 

vec[x_, n_:dim] := Table[x[i], {i, n}] *)

(*A random vector with values in the unit n-cube 

randomv[n_:dim]:= Table[Random[],{i,n}] *)

(*A random matrix of type (n,k) with elements in the unit intervall 

randommatrix[k_, n_:dim]:=Table[randomv[n], {i,k}]*)

(*A function which makes small entities to become zero.

smoothing[a_,b_:10^(-10)]:=
  Module[{c=a},If[b\[Equal]1,Return[c],If[Abs[N[c]]<b,c=0]; c,c]];
Attributes[smoothing]=Listable;
*)

(*Standard matrices *)

nullmatrix[k_Integer, n_Integer] := Table[0, {k}, {n}];
nullmatrix[n_Integer:dim] := nullmatrix[n, n]

matrix[a_,n_,m_]:= Array[a, {n, m}];
matrix[a_,n_:dim] := matrix[a, n, n]

(*Remark: We want to distinguish symbolic vectors from symbolic matrices,
and to have simple definitions, like xv = vec[x], am = matrix[a], for 
symbolic vectors and square matrices of order dim. Therefore we 
specialized the Array concept.*)

basismatrix[i_,j_,n_:dim]:=
  Table[KroneckerDelta[i,k]*KroneckerDelta[j,l],{k,n},{l,n}]

(*The commutator*)

commute[m_?MatrixQ, l_?MatrixQ] := 
	(m . l - l . m)/;(Dimensions[m] == Dimensions[l]&&
        Dimensions[m][[1]] == Dimensions[m][[2]]) 

(*The general linear Lie algebra gl(n,K) *)

glbasis[n_Integer:dim] := Module[{ord = n, i,j},
		Flatten[Table[basismatrix[i,j,ord],{i,n}, {j,n}],1]]

cobasis[i_,j_,n_:dim][m_] := 
  Sum[KroneckerDelta[i,k]*KroneckerDelta[j,l]*m[[k,l]],{k,1,n},{l,1,n}]

strconst[i_,j_,r_,s_,u_,v_,n_:dim]:= 
  cobasis[i,j][commute[basismatrix[r,s,n],basismatrix[u,v,n]]]

glkilling[a_, b_, n_:dim] :=
     
  Sum[Sum[cobasis[i, j,n][ commute[a, commute[b,basismatrix[i, j,n]]]],
        {i, 1, n}], {j, 1, n}]

zelm[z_, n_:dim] := IdentityMatrix[n]*z

(*Orthogonalization of matrix sequences *)

matrixoutzero[b_List] := Module[{v= {}, l = Length[b], k = Length[b[[1]]],i},
			For[i = 1, i<= l, i++, 
				If[Not[smoothing[b[[i]]] == nullmatrix[k]],
AppendTo[v, b[[i]]],Null[],AppendTo[v, b[[i]]]]]; Return[v]]

(*
normalize[v_,prod_:glkilling]:=
  Module[{a=Simplify[prod[v,v]],w},
    If[a\[Equal]0,w=v,w=v/Sqrt[Abs[a]],w=v/Sqrt[Abs[a]]];
    Return[w]]
*)

Options[matrixorthonorm]={matrixinnerprod\[Rule]glkilling,normed\[Rule]True,
    print\[Rule]False,neglect\[Rule]-10}

matrixorthonorm[b_List,opts___]:=
  Module[{v,w,va,vb,l,ort={},i,j,f,h,sm,aus,prod,nor,pr,bn,follow,iso,
      control},{prod,nor,pr,
        bn}={matrixinnerprod,normed,print,neglect}/.{opts}/.Options[
          matrixorthonorm];
    f[h_]:=smoothing[h,10^bn];
    sm[aus_]:=MapAll[f,aus];
    w=matrixoutzero[sm[b]];
    Label[follow];
    l=Length[w];
    If[l\[Equal]0,Goto[control]];
    For[i=1,i\[LessEqual]l,i++,
      If[Not[Simplify[sm[prod[w[[i]],w[[i]]]]]\[Equal]0],
        AppendTo[ort,w[[i]]];
        If[pr\[Equal]True,Print[w[[i]]//MatrixForm]];
        v=
          Simplify[
            sm[Table[
                w[[j]]-w[[i]]*prod[w[[i]],w[[j]]]/prod[w[[i]],w[[i]]],{j,
                  l}]]];
        w=matrixoutzero[v]; Goto[follow],Null,AppendTo[ort,w[[i]]];
        If[pr\[Equal]True,Print[w[[i]]//MatrixForm]];
        v=
          Simplify[
            sm[Table[
                w[[j]]-w[[i]]*prod[w[[i]],w[[j]]]/prod[w[[i]],w[[i]]],{j,
                  l}]]];
        w=matrixoutzero[v]; Goto[follow]]];
    Label[iso];l=Length[w];
    If[l\[Equal]0,Goto[control]];
    v=w;
    If[l\[Equal]1,AppendTo[ort,v[[1]]]; 
      If[pr\[Equal]True,Print[v[[1]]//MatrixForm]];
      Goto[control]];
    For[i=1,i\[LessEqual]l,i++,
      For[j=i+1,j\[LessEqual]l,j++,
        If[Not[Simplify[sm[prod[v[[i]],v[[j]]]]]\[Equal]0],va=v[[j]]-v[[i]];
          vb=v[[j]]+v[[i]]; 				v[[i]]=va;v[[j]]=vb;
          w=matrixoutzero[Simplify[sm[v]]];Goto[follow]]]]; 			
    ort=Join[ort,v];
    Label[control];
    l=Length[ort];
    If[l\[Equal]0,Return[ort]];
    If[nor\[Equal]True,
      ort=sm[Table[Simplify[normalize[ort[[i]],prod]],{i,Length[ort]}]]];
    For[i=1,i\[LessEqual]l-1,i++,
      For[j=i+1,j\[LessEqual]l,j++,
        If[Not[Simplify[sm[prod[ort[[i]],ort[[j]]]]]\[Equal]0],
          Return[Print["I cannot orthogonalize the given sequence!"]],Null,
          Return[Print["I cannot orthogonalize the given sequence!"]]]]]; 	
    Return[ort]]

(*The special linear Lie algebra sl(n, K) *)


slbasismatrix[i_, j_,n_:dim] :=
	basismatrix[i,j,n]/;i!= j;
slbasismatrix[i_, j_, n_:dim] := 
(basismatrix[i,j,n]-	basismatrix[n,n,n])/;(i== j&& j<n)

slbasis[n_:dim] := 
  Join[Flatten[Table[slbasismatrix[i,j,n],{i, n-1},{j, n}],1],
    Table[slbasismatrix[n,h,n], {h,n-1}]]

(*The orthogonal Lie algebra o(n) *)

obm[i_, j_,n_:dim] := basismatrix[i, j,n] - basismatrix[j, i, n]/;i < j

obasis[n_:dim]:=Flatten[Table[Table[obm[i,j,n],{j,i+1,n}],{i,n}],1]

oelm[a_,n_:dim] := Sum[Sum[obm[i, j,n] a[i, j], {i, 1, j -1}], 
                 {j, 2, n}] 

okilling[a_, b_, n_:dim] := 
  Sum[Sum[cobasis[i, j,n][ Evaluate[commute[a, commute[b, obm[i, j, n]]]]], 
    {i, 1, j - 1}], {j, 2, n}] 

(*The pseudo-orthogonal Lie algebras o(n,k) *)

psobm[i_ ,h_,k_:ind,n_:dim] :=
	
  If[i<=n-k&& h>n -k,basismatrix[i,h,n]+basismatrix[h,i,n],
 obm[ i, h,n]]/;(i<h && k <= n)

psobasis[k_:ind, n_:dim]:=
  Flatten[Table[Table[psobm[i,j,k,n],{j,i+1,n}],{i,n}],1]



psoelm[a_,k_:ind,n_:dim] := Sum[Sum[psobm[i, j,k,n] a[i, j], 
			{i, 1, j -1}], {j, 2, n}] /;k<= n



psokilling[a_, b_,k_:ind, n_:dim] := 
  Sum[Sum[
      cobasis[i, j,n][ Evaluate[commute[a, commute[b, psobm[i,j,k , n]]]]], 
    {i, 1, j - 1}], {j, 2, n}] 

(*The symplectic Lie algebras sp(2m, K) *)

alt[n_?EvenQ]:= Module[{m=nullmatrix[n], i},
		For[i= 1, i<=n/2, i++,( m[[i,i + n/2 ]] = 1; m[[i+n/2, i]]= -1;)];
		m]

altprod[x_List,y_List] := 
  x.(alt[Length[x]].y)/;(Length[x]==Length[y]&&EvenQ[Length[x]])

spbasis[i_, j_, n_?EvenQ]:= Module[{m=nullmatrix[n], k = n/2},
		If[i<=k&&j<=k, m[[i,j]] = 1; m[[k+j, k+i]] = -1;Return[m]];
		If[i<=k&&i<=j-k <=k, m[[i,j]] = 1; m[[j-k,i + k]] = 1; Return[m]];
		If[i > k&& i-k<=j <= k, m[[i,j]] = 1; m[[k+j,i-k]]= 1;Return[m]];
		Return[nullmatrix[n]]]

spbasislist[n_?EvenQ] :=  
  matrixoutzero[Flatten[Table[spbasis[i,j,n],{i,n},{j,n}],1]]

spelm[a_,n_?EvenQ] :=Sum[spbasis[i,j,n]*a[i,j],{i,n},{j,n}]

spkilling[a_, b_,n_?EvenQ] :=
     
  Sum[Sum[cobasis[i, j,n][ commute[a, commute[b,spbasis[i, j,n]]]],
        {i, 1, n}], {j, 1, n}]

End[]

Protect[ nullmatrix, 
commute, basismatrix, glbasis, cobasis,strconst, glkilling, zelm,
matrixoutzero,  matrixorthonorm, matrixinnerprod, slbasismatrix, slbasis, obm, obasis, oelm, okilling, psobm, 
psobasis, psoelm, psokilling, alt, altprod, spbasis, spbasislist, 
spelm, spkilling]


EndPackage[]
