(* mspher.m *)

(* Created by Rolf Sulanke *)

(* Started March 20, 1995 , revised Sep 24,1998, last revision February 25, 2000. *)

(*This package contains procedures and functions for working in the Moebius geometry 
of spheres. It continues the package euvec.m which contains concepts of the euclidean 
geometry of spheres. Operations and functions of pseudo-euclidean vector algebra (see 
the package pseuvec.m and the notebook pseuklid.nb) are used. For applications
and further  procedures see the notebook mspheres.nb.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(* Hint: Before importing this package be sure that the numbers
dim (= dimension) and ind (=index) of the pseudo-euclidean vector space are \
defined as constants within the Global context !!!*)

(*Remark: The functions psphere, paramsphere,showsphere are deleted now; replace them 
in an appropriate way by euklidsphere, euklidsphereplot3D. New functions: vradius,vcenter.*)

BeginPackage["mspher`",{"euvec`", "pseuvec`"}]     



spt::usage ="spt[v] projects the vector v onto the vector v/v[[dim]]-stb[dim] in  the (dim-1)-space 
orthogonal to stb[dim].";

stepro::usage ="stepro is the stereographical projection in  pseudo-euclidean
terms (ind = 1), with values in the euclidean (dim - 2)-space.Warning:
stepro[v] is correct only for isotropic vectors v!";

invstepro::usage="invstepro is the inverse of the stereographic projection stepro (ind = 1).";

plotsphere::usage = "plotsphere[v, opts] plots the sphere in the euclidean
3-space,  corresponding to the spacelike vector v of the pseudo-euclidean
5-space, after stereographical projection of the 3-sphere from its north pole
to its equatorial plane; opts are transferred
to ParametricPlot3D. plotsphere is obsolete now; use euklidsphereplot3D.";   

spunit::usage = "spunit[s,t,u,v] is a parameter representation of the manifold
of all spacelike unit vectors in the 5-dimensional pseudo-euclidean space of
index 1; the ranges are: 0<=s<=2Pi, -Pi/2<=t,u<=Pi/2, v arbitrary real."

pstg5frame::usage = "pstg5frame[s,t,u,v] is a pseudo-orthonormal moving frame
adapted to the manifold spunit[s,t,u,v] of spacelike unit vectors; the last
vector equals spunit[s,t,u,v]." 

vradius::usage = "vradius[vx] is the radius of the sphere in the euclidean 
3-space corresponding to the spacelike vector vx of the pseudo-euclidean 5-space, 
and Infinity if this sphere is a plane."

vcenter::usage = "vcenter[vx] is the center of the sphere in the euclidean 
3-space corresponding to the spacelike vector vx of the pseudo-euclidean 5-space, 
and the infinite point if this sphere is a plane."

euklidsphere::usage = "euklidsphere[vx][u,v] is a parameter representation
of the sphere (resp. plane), with geographical parameters u,v, in the euclidean
3-space, which corresponds to the spacelike vector vx." 

euklidsphereplot3D::usage = "euklidsphereplot3D[vx,u0:-1,u1:1,v0:-1,v1:1,opts] 
is the plot of a region of the sphere corresponding to the spacelike vector 
vx; the region is defined by the parameters u0, u1, v0, v1. With 
the default values the whole sphere or a square with side length 2 of the plane 
is plotted; options opts are transferred to ParametricPlot3D."

sphvec::usage = "sphvec[p1, p2, p3, p4] yields a spacelike vector of the 
pseudo-euclidean 5-space defining the sphere through the four points  p1, p2,
p3, p4 (in general position) of the euclidean 3-space.";

spacevec::usage = "spacevec[x, y, z, r] yields a spacelike vector of the 
pseudo-euclidean 5-space defining the sphere with center {x, y, z} and radius
r of the euclidean 3-space.";

gencos::usage = "gencos[v,w] is the generalized cos of the (may be imginary) 
intersection angle of the spheres defined by the spacelike vectors v,w.";

confinv::usage = "confinv[r1, r2, d] is the conformal invariant of two spheres
 with radii r1, r2 and distance d of their centers.";

planev::usage = "planev[a,b,c,p] yields a spacelike vector of the
pseudo-euclidean  5-space defining the plane with equation ax+by+cz=p.";



Begin["`Private`"]

dim = Global`dim;

ind = Global`ind;

(*Projections of isotropic vectors on sphere points*)



spt[v_List] := Module[{w,u,d=Length[v]},w =v[[d]];
			If[w===0,  Return[Print[v ," has last component ",w]],
				u=Take[v/w,d-1];Return[u]]]
				
stepro[v_List] := sterproj[spt[v]]

(*The inverse  projection: sphere points - isotropic vector*)


invstepro[v_List] := Append[invsterproj[v],1]


(* Definition of the sphere  in the Euclidean 3-Space corresponding to a spacelike vector: *)

spunit[s_,t_,u_, v_] :=
	{Cos[s] Cos[t] Cos[u] Cosh[v],Cos[t] Cos[u] Cosh[v] Sin[s],
    Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u],Sinh[v]}

pstg5frame[s_, t_, u_, v_] :=
	{{- Sin[s],Cos[s],0,0,0},
		{-Cos[s] Sin[t],- Sin[s] Sin[t],Cos[t],0,0},
		{-Cos[s] Cos[t]  Sin[u],-Cos[t]  Sin[s] Sin[u],- Sin[t] Sin[u],Cos[u],0},
		{Cos[s] Cos[t] Cos[u] Sinh[v],Cos[t] Cos[u] Sin[s] Sinh[v],
      Cos[u] Sin[t] Sinh[v],Sin[u] Sinh[v],Cosh[v]},
		{Cos[s] Cos[t] Cos[u] Cosh[v],Cos[t] Cos[u] Cosh[v] Sin[s],
      Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u],Sinh[v]}}

vradius[wx_List] := Module[{s,q, vx},
      q = pssp[wx,wx];
      If[q\[LessEqual] 0,Print[wx, " is not spacelike!"];Return[]];
      vx = wx/Sqrt[q];
      s=vx[[5]]- vx[[4]];
      If[s\[Equal]0, Return[Infinity],Return[1/s],Return[1/s]]]/;
    Length[wx] \[Equal] 5

vcenter[vx_List] := Module[{s},
      If[pssp[vx,vx]\[LessEqual] 0,Print[vx, " is not spacelike!"];Return[]];
      s=vx[[5]]- vx[[4]];
      If[s\[Equal]0, Return[{Infinity,Infinity,Infinity}]];
      Return[{vx[[1]]/s,vx[[2]]/s,vx[[3]]/s}]]/;Length[vx]\[Equal]5

euklidsphere[va_List][u_,v_] := Module[{q,},
      q = pssp[va,va];
      If[q\[LessEqual] 0, Print[va, " is not spacelike!"]; Return[]];
      If[va[[4]] == va[[5]], Goto[pla]];
      Return[sphere[vcenter[va],vradius[va]][u*Pi,v*Pi/2]];
      Label[pla];
       p = va[[5]]/Sqrt[q];
      w = Take[va,3];
      ob = orthobase[w];
      Return[ob[[3]]*p + ob[[1]]*u + ob[[2]]*v]
      ]/;Length[va]\[Equal] 5
    
euklidsphereplot3D[vn_List,u0_:-1,u1_:1, v0_:-1,v1_:1,opts___]:=
  ParametricPlot3D[Evaluate[euklidsphere[vn][u,v]],{u,u0,u1}, {v,v0,v1},opts]
    
plotsphere[nv_List, opts___] := Module[{a, p, r, m4 = nul[4], z = nul[4], 
	vx, tt = IdentityMatrix[4], u, v},
	a = pssp[nv, nv];
	If[Length[nv] != 5 || a <= 0, Return[Print[
				"Vector ", nv, ": incorrect argument!"]]];
	m4 = Take[nv, 4]; a = m4 . m4; m4 = m4/Sqrt[a];
	p = nv[[5]]/Sqrt[a]; r = Sqrt[1 - p^2];
	z = p m4; 
	tt = orthoframe[m4];
	vx[u_, v_] := z + ((tt[[1]] Cos[u] + tt[[2]] Sin[u]) Cos[v] + 
		tt[[3]] Sin[v]) r;
	ParametricPlot3D[sterproj[vx[u, v]],  {u, 0, 2 Pi}, {v, -Pi/2, Pi/2}, opts]
	]

(*Definition of a spacelike vector defining the sphere through four points of the 3e-space*)

sphvec[p1_List,p2_List,p3_List, p4_List] := Module[{v=nul[5]},
		If[Length[p1]!=3||Length[p2]!=3||Length[p3]!=3||Length[p4]!=3,
		Print["An argument does not have the correct
dimension!"]Return[]]; 	
v=Simplify[ 
        pscross[{invstepro[p1], invstepro[p2],invstepro[p3],invstepro[p4]}]];
		If[v===nul[5],Print["The points are not in general
position!"];Return[],  Return[v], Return[v]]]

(*Definition of a spacelike vector defining the sphere with given center and radius *)

spacevec[x_, y_, z_, r_] := {2 x,  2 y, 2 z, -1 - r^2 + x^2 + y^2 + z^2, 1 - r^2 + x^2 + y^2 + z^2}/(2 r) 

(*Definition of a spacelike vector defining the plane with equation ax+by+cz=p of the 3e-space*)

planev[a_,b_,c_,p_] := If[Not[a^2+b^2+c^2 === 0], Return[{a, b, c, p, p}],
Return[Print["Normal vector null is not admissible!"]]]

(*Invariants of a pair of spheres*)

gencos[v_List, w_List] := Simplify[pssp[v, w]/Sqrt[pssp[v,v] pssp[w,w]]]

confinv[r1_, r2_, d_] := (r1^2 + r2^2 -d^2)/(2 r1 r2)

End[]

Protect[spt, stepro, invstepro, plotsphere, spunit,euklidsphere, euklidsphereplot3D, 
pstg5frame, sphvec, spacevec, gencos, confinv, planev, vradius, vcenter]

EndPackage[]
