(* pseuvec.m *)

(* The package contains vector  operations for the n-dimensional
pseudoeuclidean space of arbitrary index,  including the Euclidean case, 
it needs euvec.m. For application, examples etc see the notebook 
pseuklid.nb.*)

(* Created by Rolf Sulanke *)

(* Started March 18, 1995,last revision February 18, 2000. *)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(* Hint: Before importing this package be sure that the numbers
dim (= dimension) and ind (=index) of the pseudo-euclidean vector space are \
defined as constants within the Global context !!!*)

(* WARNINGS. 
1. In certain cases the pseudo-euclidean orthonormalization procedure orthonorm 
cannot  be carried out. Then instead of the orthonormed matrix appears the 
message  "I cannot orthogonalize the given sequence!"  Sometimes the norming 
is the reason; always try the orthogonalization with the option normed -> False. 
Besides of the parameter dependence the reason can be a numerical badly 
conditioned matrix b (e.g. contains vectors with very small, or very different 
in magnitude, components). In this case one can try applying orthonorm  
on the renormed matrix renorm[b]. (See the examples in the notebook pseuklid.nb.)
2. In the pseudo-euclidean case the isotropic vectors are orthogonal to \
itself. By our definition, they are considered as being normed. Therefore in \
the resulting orthogonal or orthonormal sequence there may appear linearely \
dependent isotropic vectors.
3. In the pseudo-euclidean vector spaces exist vector sequences which may not be 
orthogonalized preserving its flag structure. In difference to Erhard Schmidt's 
orthogonalization the procedure orthonorm does not preserve the flag structure 
of the given sequence in general.
*)

BeginPackage["pseuvec`",{"euvec`"}] 
   
(* The usages *)
ide::usage = "ide[d:dim, nd:ind] is a vector of length dim with first dim - \
\
ind
entries 1, and last  ind entries -1; dim  and ind must be defined
first.";

pssp::usage = "pssp[v, w,d:dim, nd:ind] is the pseudo-euclidean scalar
product of index ind and dimension dim;  dim  and ind must be defined \
\
first.";

dual::usage = "dual[v, d:dim, nd:ind] is the dual vector to v with respect
to the scalar product pssp;dim  and ind must be defined first.";

pscross::usage = "pscross[vec]  gives the vector product of a
list vec of d - 1 vectors of dimension d of the pseudo-Euclidean space; 
dim d and ind should be defined first.";

psCross::usage = "psCross[vec(1),..., vec(d-1)]  gives the 
vector product of d - 1 vectors v(i) of dimension d of the
pseudo-Euclidean space;  dim d and ind should be defined first.";

ch::usage = "ch[v] gives 1, if v is spacelike, 0, if v is lightlike, 
and -1, if v is timelike.";

pr::usage = "pr[v,x] is the projection of the vector x onto the 
hypersubspace orthogonal to the vector v with respect to the pseudo-euclidean
 scalar product pssp; pssp[v,v] != 0 is supposed."   

orthonorm::usage = "orthonorm: If b is a list of vectors in the n-dimensional pseudo-euclidean 
space of index  k, then orthonorm[b,opts] is an orthogonal list of 
vectors with the same span as span[b]; orthonorm[b] may contain linearely
dependent isotropic vectors. orthonorm can be applied to finite dimensional \
\
vector spaces and 
symmetric bilinear forms prod using the option innerprod -> prod; 
the default is prod = pssp."

normalize::usage="normalize[v, prod:pssp] is the proportional vector of the \
same
direction  with prod[v,v] = 0, 1,or -1; in case prod[v,v] = 0 one gets
normalize[v] = v." 

psfilter::usage = "psfilter[blist] permutes the vectors of the
list blist; \ the spacelike vectors are put at the first places. Besides of
this, their order and the order of the other vectors remain unchanged."

orthopair::usage = "orthopair[v] constructs a pair of orthonormalized
spacelike vectors starting from  the given spacelike vector v; dim  and ind must
be defined first.";


Begin["`Private`"]
(* Definition of the Scalar Product  *)
dim := Global`dim;
ind := Global`ind;
ide[d_:dim, nd_:ind] := Join[Table[1, {d - nd}], Table[-1, {nd}]]
dual[l_List, d_:dim, nd_:ind] := ide[d, nd] * l /; Length[l] == d
pssp[l_List, k_List, d_:dim, nd_:ind]  := (ide[d,nd] * l) . k 
SetAttributes[pssp, Orderless]
pssp[k_ + l_, m_] := pssp[k, m] + pssp[l, m]
pssp[a_ k_, l_] := a pssp[k, l] 
pssp[k_ + l_, m_] := pssp[k, m] + pssp[l, m]
 
pssp[ k_ a_, l_] := a pssp[k, l]
ch[v_List]:=Module[{p,q, v1, v2},
	If[Length[v] != dim || v == nul[], 
		Return[Print[v, " is not a correct argument!"]]];
	p=smoothing[Simplify[Evaluate[pssp[v,v]]]]; 
	If[p == 0., q = 0; Return[q]];
	v1 = Sum[stb[i]*v[[i]], {i,dim - ind}]; 
	v2 = Sum[stb[j]*v[[j]], {j, dim - ind +1, dim}];
	If[v1 === nul[] , q = -1; Return[q]];
	If[v2 === nul[], q = 1; Return[q], q = Sign[p]]; 
	If[ q ===1 || q === -1 || q === 0, Return[q], Return[Print["ch[",v,"]cannot \
\
be evaluated!"]]]]

normalize[v_, prod_:pssp]:=Module[{a=Simplify[prod[v,v]], w},
			If[a == 0,w=v,w=v/Sqrt[Abs[a]],w=v/Sqrt[Abs[a]]];
			Return[w]]
			
(* Here are the general cross products in the pseudo-Euclidean case: *)

pscross = Composition[dual, cross]

psCross = Composition[dual, Cross]

(* Orthogonal projection *)
pr[v_, x_] := x - v*pssp[x,v]/pssp[v,v]



(*A pseudo-euclidean orthonormalization procedure. *)

Options[orthonorm] = {innerprod ->pssp, normed -> True, print
->False,  neglect-> -10}


orthonorm[b_List,opts___]:=
  Module[{v,w,va,vb,l,ort={},i,j,f,h,sm,aus,prod,nor,pr,bn,follow,iso,
      control},{prod,nor,pr,
        bn}={innerprod,normed,print,neglect}/.{opts}/.Options[orthonorm];
    f[h_]:=smoothing[h,10^bn];
    sm[aus_]:=MapAll[f,aus];
    w=outzero[sm[b]];
    Label[follow];
    l=Length[w];
    If[l\[Equal]0,Goto[control]];
    For[i=1,i\[LessEqual]l,i++,
      If[Not[Simplify[sm[prod[w[[i]],w[[i]]]]]\[Equal]0],
        AppendTo[ort,w[[i]]];
        If[pr\[Equal]True,Print[w[[i]]]];
        v=
          Simplify[
            sm[Table[
                w[[j]]-w[[i]]*prod[w[[i]],w[[j]]]/prod[w[[i]],w[[i]]],{j,
                  l}]]];
        w=outzero[v]; Goto[follow],Null,AppendTo[ort,w[[i]]];
        If[pr\[Equal]True,Print[w[[i]]]];
        v=
          Simplify[
            sm[Table[
                w[[j]]-w[[i]]*prod[w[[i]],w[[j]]]/prod[w[[i]],w[[i]]],{j,
                  l}]]];
        w=outzero[v]; Goto[follow]]];
    Label[iso];l=Length[w];
    If[l\[Equal]0,Goto[control]];
    v=w;
    If[l\[Equal]1,AppendTo[ort,v[[1]]]; If[pr\[Equal]True,Print[v[[1]]]];
      Goto[control]];
    For[i=1,i\[LessEqual]l,i++,
      For[j=i+1,j\[LessEqual]l,j++,
        If[Not[Simplify[sm[prod[v[[i]],v[[j]]]]]\[Equal]0],va=v[[j]]-v[[i]];
          vb=v[[j]]+v[[i]]; 				v[[i]]=va;v[[j]]=vb;
          w=outzero[Simplify[sm[v]]];Goto[follow]]]]; 			ort=Join[ort,v];
     Label[control];
    l=Length[ort];
    If[l\[Equal]0,Return[ort]];
    If[nor\[Equal]True,
      ort=sm[Table[Simplify[normalize[ort[[i]],prod]],{i,Length[ort]}]]];
    For[i=1,i\[LessEqual]l-1,i++,
      For[j=i+1,j\[LessEqual]l,j++,
        If[Not[Simplify[sm[prod[ort[[i]],ort[[j]]]]]\[Equal]0],
          Return[Print["I cannot orthogonalize the given sequence!"]],Null,
          Return[Print["I cannot orthogonalize the given sequence!"]]]]]; 	
    Return[ort]]


(*psfilter: rearranging the sequence: spacelike vectors ahead! *)

psfilter[b_List] := Module[{i, lb = Length[b], bn= {},qn = {}},
		For[i=1,i<=lb,i++,
			If[Length[b[[i]] ] != dim, 
        Return[ Print["Incorrect argument: ", b[[i]],"!!!"]]]];
		For[i=1, i<=lb, i++,
			If[ch[b[[i]] ]== 1, AppendTo[bn, b[[i]]],AppendTo[qn,b[[i]]],
        AppendTo[qn,b[[i]]]]];
				Return[Join[bn,qn]]]

(* Now we generate a pair of orthonormal spacelike vectors starting from a \
\
given spacelike one; 
that we need for the geometry of circles on the 3-sphere. *)

orthopair[l_List] := 
Module[{a , l1 = l, m1 = nul[],b = dim - ind, j, k},
a = Simplify[pssp[l, l]];
If[a <= 0, Print["Vector ",l," is not spacelike!"];Return[], Null,
Print["Maybe, Vector ",l," is not spacelike?"];Return[]]; 
If[ b < 2, Print["Orthogonal spacelike pairs don't exist!"];Return[]]; 	 
l1 = l1/Sqrt[a]; 	
	 	For[k = 1, k <= b, k++, 			
	If[l1[[k]] == 0, m1 = stb[k]; Return[{l1, m1}]]]; 	 
		For[j = 1, j <= b, j++, 		 	
If[OddQ[j]&& j< b, m1[[j]] = - l1[[j + 1]]];  		
	If[EvenQ[j], m1[[j]] = l1[[j - 1]]]]; 		
		m1 = m1/Sqrt[pssp[m1, m1]];  				
{l1, m1}]/; Length[l] == dim

End[]	
Protect[pssp, ide, psCross, pscross, ch, dual, pr, psfilter,
orthonorm, normalize, orthopair]
EndPackage[]

